clear
format long



%----------------------------Comments--------------------------------------
%
%
%In the code below ``the present work'' always referes to the paper:
%
%``Fourier-Taylor approximation of unstable manifolds for compact
%  maps: numerical implementation and computer assisted error bounds'' 
%  by JDMJ.
%
%This manuscript describes the theoretical justification of the
%computations implemented below.  All computations are for the Kot-Schaffer
%model discussed in the present work.
%
%
%
%--------------------------------------------------------------------------

%PROGRAM DEPENDENCIES:  this program depends on the programs
%
% --
% --
% --
% --
% --
% --
% --
%
%
%The program also uses the IntLab library for interval arithmetic.
%We assume that IntLab has been started before the current program
%is called.  It this is not true then the program will crast the 
%first time an intlab command is encountered.
%
%


%==========================================================================
%User Controls:
%==========================================================================


N = 26          %-- number of Fourier-cosine modes (spatial discretization)
M = 5           %-- number of Taylor modes (manifold discretization)

%-- `mu' is the reproductive rate in Kot-Schaffer model
%    we use an interval version and a floating point version

mu_int = intval(35)/intval(10); 
mu = mid(mu_int);        

%-- `nu' is the weight associated with the "ell nu one" norm
%    We only use an interval version
nu = intval(11)/intval(10);

%-- The parameter ``scaleEig'' determines the scaling of the unstable 
%   eigenvector used in the manifold computation.  This and the Taylor 
%   order M are the main ``dials'' used to control the manifold computation

scaleEig = 0.1



%A cosmetic variable which determines the number of times 
%to iterate to generate the view of the attractor
orbitPoints = 10000;



%INITIALIZE SOME CONTAINERS:
%Trivial Fixed Point:
p0 = zeros(N+1, 1);
%Smoothing Operator:
b = zeros(N+1, 1);
 
%c^{-1} coefficients:
c = zeros(N+1, 1);

%An initial condition:
a = zeros(N+1, 1);
a(1) = 1;

%Initialize smooting operator and initial condition:
for n = 0:N  
    b(n+1) = 2^(-n);
    c(n+1) = 0.5*5^(-n);
end


%==========================================================================
%   Start the computations:
%==========================================================================


%-----------Compute a Nontrival Fixed Point by a Newton Method:------------
%  The discretized Kot-Schaffer map and its discretized differential
%  are defined/computed in Equations (20) and (21) of the present
%  work.  Both are evaluated via interval FFT

a0 = zeros(N+1, 1);
a0(1) = 1;

a_now = a0;
I_N = eye(N+1);
for n = 1:8
   
   DG =  I_N - kotSchafferDifferential_logistic(a_now, b, c, mu, N);
   DG_inv = inv(DG);
   Ga = a_now - kotSchaffer_logistic(a_now, b, c, mu, N);
   a_now = a_now - DG_inv*Ga;
   defect = norm(a_now -  kotSchaffer_logistic(a_now, b, c, mu, N), 1);
end


disp('the approximate non-trivial fixed point is:')
p1 = a_now


aN = intval(p1);

b_int = intval(zeros(N+1, 1));
c_int = intval(zeros(N+1, 1));
b_tail_int = intval(zeros(3*N+1, 1));
c_tail_int = intval(zeros(3*N+1, 1));


%Initialize smooting operator and initial condition (interval versions):
for n = 0:N
    b_int(n+1) =  intval(2)^(-n); 
    c_int(n+1) = (intval(1)/intval(2))*intval(5)^(-n); 
end

for n = 0:3*N
    b_tail_int(n+1) =  intval(2)^(-n); 
    c_tail_int(n+1) = (intval(1)/intval(2))*intval(5)^(-n); 
end

%Below are the ell-nu-one bounds on the tails of the 
%sequences b, c.  They are needed in the truncation error 
%bounds and also needed to evaluate the norms of b, c.
%See Section 2.1.1 of the present work, especially equation 
%number 6.

c_inf_int = ((nu/intval(5))^(N+1))/(2*(1 - (nu/5)))
b_inf_int = ((nu/4)^(N+2))/(1 - (nu/4))

%==========================================================================
%  The next several sections compute the first order data along with 
%  vaildated error bounds.  These computations are as discussed in the 
%  appendices of the paper.  
%==========================================================================


%--------------------Fixed Point Validation -------------------------------
%
% The program ``kotSchaffer_validateFP'' called below carries out the 
%analytis discussed in Appendix A and B of the present work.  

disp('validating the fixed point computation...')

[r_minus, r_plus, proofGood] = ...
     kotSchaffer_validateFP(...
          aN, mu_int, nu, b_int, b_tail_int, ...
          c_int, c_tail_int, b_inf_int, ...
          c_inf_int, N);

if proofGood == 0      
      crashNow = badFixedPointComputation
end
      
disp('the fixed point is validated!') 
disp('the validated bounds are:')      
r0 = r_minus



%---------------Eigenvalue/Eigenvector Computation -------------------------
%
%This seciton of code carries out the validations described in Appendix C
%of the present work.  

%First we fit the eigenvalue/eigenvector data to the fixed point 
%operator of appendic C by running the Newton scheme.  Typically 
%this moves the eigendata hardly at all, as the numerical computation
%of the eigenvalues/eigenvectors in MatLab is already close to 
%machine precision.  Indeed this step is not strictly speaking necessary



%----First: compute the first order data, i.e. the eigenvalues and 
%    eigenvectors.  Validated finite dimensional computations in IntLab

%Linearization at the trivial fixed point (floating point):
D = kotSchafferDifferential_logistic(p1, b, c, mu, N);

%It's eigenvalues and eigenvectors:
[Q, Sigma] = eig(full(D));


%Interval enclosure of the truncated derivative:
D_intval = kotSchafferDifferential_logistic_intval(...
    aN, b_int, c_int, mu_int, N);

%Verrify the eigendata for D_intval (IntLab)
Q_intval = intval(zeros(N+1, N+1));
Sigma_intval = intval(zeros(N+1, N+1));
for n = 0:N
   [Sigma_intval(n+1, n+1), Q_intval(:, n+1)] = ...
         verifyeig(D_intval, Sigma(n+1,n+1), Q(:, n+1)); 
end

Q_inv_intval = inv(Q_intval);


spectrum_DFp = diag(Sigma);

%Our unstable approximate eigenvalue:
lambda = spectrum_DFp(1);

%Our unstable approximate eigenvector:
xi = scaleEig*Q(:,1);

%Define the ``phase condition'' for the eigenvector
% (see equation 55 and 56 of Appendix C)
s = intval(0);
for n = 0:N
   s = s + xi(n+1); 
end

%Iterate the Newton map. The discretized eigenvalu map and its differential
%are defined by Equation (57) andjust before Equation (59) respectively.
for n = 1:3
    E = kotEigMap_cos(p1, b, c, xi, lambda, mid(s), mu, N);
    DE = kotEigMap_Differential_cos(p1, b, c, xi, lambda, mu, N);
    Delta = -DE\E;
    y = [lambda; xi];
    y = y + Delta; 
    lambda = y(1);
    xi = y(2:end);   
end


disp('approximate unstable eigenvector')
xi_int = intval(xi)
disp('approximate unstable eigenvalue')
lambda_int = intval(lambda)

%The program ``kotSchaffer_validateEig'' computes the constants defined in 
%Lemma C.1 of the present work and checks the hypotheses of that Lemma.

disp('validating the eigenvalue/eigenvector pair')
[r_minus, r_plus, proofGood] = kotSchaffer_validateEig(...
          aN, lambda_int, xi_int, s, mu_int, nu, ...
          b_int, b_tail_int, c_int, c_tail_int, ...
          b_inf_int, c_inf_int, r0, N);

if proofGood == 0
    crashNow = badEigendataValidation
end
      
      
disp('the eigendata is validated!')
disp('validated error bounds for the linear data')
r1 = r_minus


%--------------------------------------------------------------------------
%Prove that there is exactly one unstable eigenvalue at p1.  The code
%caries out the check described in Appendix F of the present work.

disp('validating the eigenvalue count')
eigCheck = kot_oneUnstableEig_check(...
    aN, r0, Sigma, Q, b_int, b_tail_int, c_int, c_inf_int, ...
    mu_int, nu, N)

if eigCheck == 0
    crashNow = badEigCheck
end

disp('count is correct!')


%==========================================================================
%==========================================================================
%==========================================================================
%==========================================================================
%==========================================================================
%   VALIDATE THE UNSTABLE MANIFOLD APPROXIMATION: 
%
%   This is the main computation of the paper.  The program below computes
%   the constants defined the lemmas of Sction 4.4 and then checks the 
%   conditions of the Newton-Kantorovich theorem (Thm 2.2)
%
%   If the validation is successful (i.e. if the hypothese of the lemmas 
%   are satisfied) then the program returns the success and the validated
%   error bounds.
%
%==========================================================================
%==========================================================================
%==========================================================================
%==========================================================================
%==========================================================================


%===============  Compute the Unstable Maniofld ===========================
%Initialize the Parameterization Container:

disp('computing the approximate unstable parameterization...')
PMN = computeKotSchafferManifold_1D(...
    p1, lambda, xi, M, N, b, c, mu, mid(nu));


coeffNorms = zeros(1, M+1);
for m = 0:M
   thisCoeff = PMN(:, m+1);
   thisNorm = ellOneNorm(thisCoeff, mid(nu), N);
   coeffNorms(m+1) = thisNorm;
end

coefAxis = linspace(0, M, M+1)

figure 
hold on 
plot(coefAxis, coeffNorms, 'b')
plot(coefAxis, coeffNorms, 'k*')

figure 
hold on 
plot(coefAxis, log(coeffNorms)/log(10), 'b')
plot(coefAxis, log(coeffNorms)/log(10), 'k*')



%The validation values (Definition 4.5) have to be passed into
%the validation function.  The following script checks that 
%the 1, 2, 5 of the definition are satisfied.  That 3 is satisfied 
% has to be checked by hand (this uses the bound given by Equation (8)
% in the paper).  That 6 is satisfied is assured as long as the 
%fixed point validation above succedes.  That 7 is satisfied is assured
%as long as the eigenvalue/eigenvector validation above succedes.
%8 is checked internally to the "validateUnstableManifold_Kot1D_fast"
%function below, as that is where the approximate inverse operator 
%is computed.  This is the reason for the "so far" in the report below.

disp('check validation values...')

%Check the validation values:
valValsGood = 1;
if inf(nu) <= 1
    valValsGood = 0
elseif inf(mu_int) <= 0
    valValsGood = 0
elseif inf(abs(lambda_int)^(M+1) - abs(mu_int)*abs(b_tail_int(N+2))) <= 0
    valValsGood = 0
end

if valValsGood == 0
    crashNow = badValVals
end
    
disp('the validation values are good! (so far...)')

disp('validating the unstable manifold approximation...')
tic


[R_minus, R_plus, parmGood] = validateUnstableManifold_Kot1D_fast(...
                    aN, r0, r1, PMN, xi_int, lambda_int, ...
                    b_int, b_tail_int, b_inf_int, ...
                    c_int, c_tail_int, c_inf_int, ...
                    mu_int, nu, N, M)

proofTime_wu = toc             

if parmGood == 0
    crashNow = badParmValidation
end

disp('the parameterization is validated!')

disp('The validated parameterization bounds are:')
r_P = sup(R_minus)



%--------------------------------------------------------------------------
% Plot the Unstable Manifold:
%--------------------------------------------------------------------------

numPoints = 1500;
parmDomain = linspace(-1, 1, numPoints);
Wu1_loc = zeros(N+1, numPoints);

for k = 1:numPoints
    Wu1_loc(:, k) = evaluate_P1D(parmDomain(k), PMN, M, N);
end

figure
hold on

plot3(Wu1_loc(1, :), Wu1_loc(2, :), Wu1_loc(3, :), 'b')
plot3(p1(1), p1(2), p1(3), 'k*')
plot3(0, 0, 0, 'k*')
axis([p1(1)-1.5 p1(1)+1.5 p1(2)-0.25 p1(2)+0.25 p1(3)-0.1 p1(3)+0.1])    
  






