clear
format long



%----------------------------Comments--------------------------------------
%
%
%In the code below ``the present work'' always referes to the paper:
%
%``Fourier-Taylor approximation of unstable manifolds for compact
%  maps: numerical implementation and computer assisted error bounds'' 
%  by JDMJ.
%
%This manuscript describes the theoretical justification of the
%computations implemented below.  All computations are for the Kot-Schaffer
%model discussed in the present work.
%
%
%
%--------------------------------------------------------------------------

%PROGRAM DEPENDENCIES:  this program depends on the programs
%
% --
% --
% --
% --
% --
% --
% --
%
%
%The program also uses the IntLab library for interval arithmetic.
%We assume that IntLab has been started before the current program
%is called.  It this is not true then the program will crast the 
%first time an intlab command is encountered.
%
%


%==========================================================================
%User Controls:
%==========================================================================


N = 26          %-- number of Fourier-cosine modes (spatial discretization)

%-- `mu' is the reproductive rate in Kot-Schaffer model
%    we use an interval version and a floating point version

mu_int = intval(35)/intval(10); 
mu = mid(mu_int);        

%-- `nu' is the weight associated with the "ell nu one" norm
%    We only use an interval version
nu = intval(11)/intval(10);


%A cosmetic variable which determines the number of times 
%to iterate to generate the view of the attractor
orbitPoints = 10000;


scaleEig = 1

%INITIALIZE SOME CONTAINERS:
%Trivial Fixed Point:
p0 = zeros(N+1, 1);
%Smoothing Operator:
b = zeros(N+1, 1);
 
%c^{-1} coefficients:
c = zeros(N+1, 1);

%An initial condition:
a = zeros(N+1, 1);
a(1) = 1;

%Initialize smooting operator and initial condition:
for n = 0:N  
    b(n+1) = 2^(-n);
    c(n+1) = 0.5*5^(-n);
end


%==========================================================================
%   Start the computations:
%==========================================================================


%-----------Compute a Nontrival Fixed Point by a Newton Method:------------
%  The discretized Kot-Schaffer map and its discretized differential
%  are defined/computed in Equations (20) and (21) of the present
%  work.  Both are evaluated via interval FFT

a0 = zeros(N+1, 1);
a0(1) = 1;

a_now = a0;
I_N = eye(N+1);
for n = 1:8
   
   DG =  I_N - kotSchafferDifferential_logistic(a_now, b, c, mu, N);
   DG_inv = inv(DG);
   Ga = a_now - kotSchaffer_logistic(a_now, b, c, mu, N);
   a_now = a_now - DG_inv*Ga;
   defect = norm(a_now -  kotSchaffer_logistic(a_now, b, c, mu, N), 1);
end


disp('the approximate non-trivial fixed point is:')
p1 = a_now


aN = intval(p1);

b_int = intval(zeros(N+1, 1));
c_int = intval(zeros(N+1, 1));
b_tail_int = intval(zeros(3*N+1, 1));
c_tail_int = intval(zeros(3*N+1, 1));


%Initialize smooting operator and initial condition (interval versions):
for n = 0:N
    b_int(n+1) =  intval(2)^(-n); 
    c_int(n+1) = (intval(1)/intval(2))*intval(5)^(-n); 
end

for n = 0:3*N
    b_tail_int(n+1) =  intval(2)^(-n); 
    c_tail_int(n+1) = (intval(1)/intval(2))*intval(5)^(-n); 
end

%Below are the ell-nu-one bounds on the tails of the 
%sequences b, c.  They are needed in the truncation error 
%bounds and also needed to evaluate the norms of b, c.
%See Section 2.1.1 of the present work, especially equation 
%number 6.

c_inf_int = ((nu/intval(5))^(N+1))/(2*(1 - (nu/5)))
b_inf_int = ((nu/4)^(N+2))/(1 - (nu/4))

%==========================================================================
%  The next several sections compute the first order data along with 
%  vaildated error bounds.  These computations are as discussed in the 
%  appendices of the paper.  
%==========================================================================


%--------------------Fixed Point Validation -------------------------------
%
% The program ``kotSchaffer_validateFP'' called below carries out the 
%analytis discussed in Appendix A and B of the present work.  

disp('validating the fixed point computation...')

[r_minus, r_plus, proofGood] = ...
     kotSchaffer_validateFP(...
          aN, mu_int, nu, b_int, b_tail_int, ...
          c_int, c_tail_int, b_inf_int, ...
          c_inf_int, N);

if proofGood == 0      
      crashNow = badFixedPointComputation
end
      
disp('the fixed point is validated!') 
disp('the validated bounds are:')      
r0 = r_minus



%---------------Eigenvalue/Eigenvector Computation -------------------------
%
%This seciton of code carries out the validations described in Appendix C
%of the present work.  

%First we fit the eigenvalue/eigenvector data to the fixed point 
%operator of appendic C by running the Newton scheme.  Typically 
%this moves the eigendata hardly at all, as the numerical computation
%of the eigenvalues/eigenvectors in MatLab is already close to 
%machine precision.  Indeed this step is not strictly speaking necessary



%----First: compute the first order data, i.e. the eigenvalues and 
%    eigenvectors.  Validated finite dimensional computations in IntLab

%Linearization at the trivial fixed point (floating point):
D = kotSchafferDifferential_logistic(p1, b, c, mu, N);

%It's eigenvalues and eigenvectors:
[Q, Sigma] = eig(full(D));


%Interval enclosure of the truncated derivative:
D_intval = kotSchafferDifferential_logistic_intval(...
    aN, b_int, c_int, mu_int, N);

%Verrify the eigendata for D_intval (IntLab)
Q_intval = intval(zeros(N+1, N+1));
Sigma_intval = intval(zeros(N+1, N+1));
for n = 0:N
   [Sigma_intval(n+1, n+1), Q_intval(:, n+1)] = ...
         verifyeig(D_intval, Sigma(n+1,n+1), Q(:, n+1)); 
end

Q_inv_intval = inv(Q_intval);


spectrum_DFp = diag(Sigma);

%Our unstable approximate eigenvalue:
lambda = spectrum_DFp(1);

%Our unstable approximate eigenvector:
xi = scaleEig*Q(:,1);

%Define the ``phase condition'' for the eigenvector
% (see equation 55 and 56 of Appendix C)
s = intval(0);
for n = 0:N
   s = s + xi(n+1); 
end

%Iterate the Newton map. The discretized eigenvalu map and its differential
%are defined by Equation (57) andjust before Equation (59) respectively.
for n = 1:3
    E = kotEigMap_cos(p1, b, c, xi, lambda, mid(s), mu, N);
    DE = kotEigMap_Differential_cos(p1, b, c, xi, lambda, mu, N);
    Delta = -DE\E;
    y = [lambda; xi];
    y = y + Delta; 
    lambda = y(1);
    xi = y(2:end);   
end


disp('approximate unstable eigenvector')
xi_int = intval(xi)
disp('approximate unstable eigenvalue')
lambda_int = intval(lambda)

%The program ``kotSchaffer_validateEig'' computes the constants defined in 
%Lemma C.1 of the present work and checks the hypotheses of that Lemma.

disp('validating the eigenvalue/eigenvector pair')
[r_minus, r_plus, proofGood] = kotSchaffer_validateEig(...
          aN, lambda_int, xi_int, s, mu_int, nu, ...
          b_int, b_tail_int, c_int, c_tail_int, ...
          b_inf_int, c_inf_int, r0, N);

if proofGood == 0
    crashNow = badEigendataValidation
end
      
      
disp('the eigendata is validated!')
disp('validated error bounds for the linear data')
r1 = r_minus


%--------------------------------------------------------------------------
%Prove that there is exactly one unstable eigenvalue at p1.  The code
%caries out the check described in Appendix F of the present work.

disp('validating the eigenvalue count')
eigCheck = kot_oneUnstableEig_check(...
    aN, r0, Sigma, Q, b_int, b_tail_int, c_int, c_inf_int, ...
    mu_int, nu, N)

if eigCheck == 0
    crashNow = badEigCheck
end

disp('count is correct!')




%------------------- Plotting ---------------------------------------------



%Function Space:
u0 = zeros(1, 500);
cx = zeros(1, 500);
bx = zeros(1, 500);
xix = zeros(1, 500);
xAxis = linspace(0, pi, 500);
for n = 1:500
  x = xAxis(n);  
  u0(n) = evaluateCosineSeries(x, p1, N);
  cx(n) = evaluateCosineSeries(x, c, N);
  bx(n) = evaluateCosineSeries(x, b, N);
  xix(n) = evaluateCosineSeries(x, xi, N);
end

figure 
hold on
plot(xAxis, u0, 'b')
plot(xAxis, cx, 'g')
plot(xAxis, bx, 'r')
plot(xAxis, xix, 'k')



%Decay Rates:
figure 
hold on

coefAxis = linspace(0, N, N+1);
plot(coefAxis, log(abs(p1))/log(10), 'b.')
plot(coefAxis, log(abs(c))/log(10), 'g.')
plot(coefAxis, log(abs(b))/log(10), 'r.')
plot(coefAxis, log(abs(xi))/log(10), 'k.')






