function [r_minus, r_plus, proofGood] = kotSchaffer_validateEig(...
          aN, lambda, xi, s, mu, nu, ...
          b, b_tail, c, c_tail, b_inf, c_inf, r0, N);


%--------------------------------------------------------------------------
%
%Note: In this file ``the paper'' refers to 
%
%      Fourier-Taylor Approximation of Unstable Manifolds for Compact Maps: 
%          Numerical Implementation and Computer Assisted Error Bounds
%
%
%
%This program implements the validation discussed in Appendix C of the
%paper.  The purpose of the program is simply to check that the hypotheses
%of the lemma are satisfied.  If they are then the program reports that
%the proof has succeded via proofGood = 1, and returns the validated error
%bounds.  These are bounds on the roots of the radius polynomial defined 
%in Theorem C.1.
 

%---- Validation: ---------------------------------------------------------

     
%The ell-nu-one norm of the numerical approximation      
xi_norm = ellOneNorm_intval(xi, nu, N);

%The differential of the eigenvalue/eigenvector operator 
%defined in Equation (57) and just before Equation (59)
%of Appendix C.

DE = kotEigMap_Differential_cos_intval(...
                           aN, b, c, xi, lambda, mu, N);

%Numerical approximate inverse as used in definition of 
%equation (60).
AN = inv(mid(DE));                       
AN = intval(AN);

%Diagonal matrix for b^N:
BN = intval(eye(N+2));
for n = 0:N
   BN(n+2, n+2) = b(n+1); 
end

%Matrix Norms of A and A*B:
in_matrixNorm = 0;
A_norm = R_EllOneMatrixNorm_intval(AN, 1/abs(lambda), nu, N);
ANBN_norm = R_EllOneMatrixNorm_intval(...
    AN*BN, abs(b_tail(N+2))/abs(lambda), nu, N);
in_matrixNorm = 1;

%Numerical defect:
E = kotEigMap_cos_intval(aN, b, c, xi, lambda, s, mu, N);
%Numerical defect pre-cionditioned by A:
Defect = AN*E;

%First term of Y_0 from Lemma C.1:
Y0 = R_EllOneNorm_intval(Defect, nu, N);


%Prepare to compute the second term of Y_0.  Need the 
%numerical convolutions to order 3N.  These are computed using 
%the FFT:
cNBig = intval(zeros(3*N+1,1));
aNBig = intval(zeros(3*N+1,1));
xiNBig = intval(zeros(3*N+1,1));
cNBig(1:N+1) = c;
aNBig(1:N+1) = aN;
xiNBig(1:N+1) = xi;


cNaNaN = int_cubic_sumFFT(cNBig,xiNBig,aNBig);

%Compute the second term of Y_0:
tailExponents = (N+1:3*N);
nu_tail = nu.^tailExponents;
tempTail = abs(b_tail(N+2:3*N+1).*cNaNaN(N+2:3*N+1).*nu_tail');
sum = intval(ones(1, 2*N))*tempTail;

%Add to the first term:
Y0 = Y0 + 4*(abs(mu)/abs(lambda))*sum;

%Build the third term of Y_0:
a_norm = ellOneNorm_intval(aN, nu, N);
b_N_norm = ellOneNorm_intval(b, nu, N);
b_norm = b_N_norm + b_inf;
c_N_norm = ellOneNorm_intval(c, nu, N);
c_norm = c_N_norm + c_inf;

%Final computation of Y_0 from Lemma C.1:
Y0 = Y0 + 2*abs(mu)*xi_norm*ANBN_norm*(c_norm*r0 + c_inf*(a_norm+r0));

%Compute the Z_0 bound as defined in Lemma C.1:
Id = intval(eye(N+2));
Z0 = R_EllOneMatrixNorm_intval(Id - AN*DE, intval(0), nu, N);



%Prepar to compute the Z_1 bounds.  First require the ``kappa bounds''
%of Lemma 3.3.  These are computed by the following function:
kappas = kappaBounds(c, aN, nu, N);

%Compute the first term of Z_1 from Lemma C.1:
sum = intval(0);
for k = 0:N
   sum = sum +  abs(AN(1, k+2))*abs(b(k+1))*kappas(k+1);
end

%Compute the second term of Z_1 (the double sum) from Lemma C.1:
for n = 0:N
    for k = 0:N
        sum = sum + ...
            abs(AN(n+2, k+2))*abs(b(k+1))*...
            kappas(k+1)*nu^n;
    end
end

%Compute the last three terms of Z_1 from Lemma C.1:
Z1 = 4*abs(mu)*sum;
Z1 = Z1 + 2*abs(mu)*ANBN_norm*c_norm*r0;
Z1 = Z1 + 2*abs(mu)*ANBN_norm*c_inf*(a_norm + r0);

%Define the Z_1 bound as in Lemma C.1:
Z1 = Z1 + (abs(mu)*abs(b_tail(N+2))/abs(lambda))*...
    (1 + 2*c_norm*(a_norm + r0));

%Define the Z_2 bound as in Lemma C.1:
Z2 = A_norm;

%Define and solve the ``radius polynomial'' from Lemma C.1.
a_quad = Z2;
b_quad = -(1 - Z0 - Z1);
c_quad = Y0;

%The root r_minus gives the smallest error bound on the 
%validated eigenvalue and eigenvector:
r_minus = (-b_quad - sqrt(b_quad^2 - 4*a_quad*c_quad))/(2*a_quad);

%r_plus gives an isolation bound (for the eigenvector problem there 
%are exactly two solutions to the global problem and the differ only 
%by the sign, again this is true only after the phase condition is 
%fixed)



r_plus = (-b_quad + sqrt(b_quad^2 - 4*a_quad*c_quad))/(2*a_quad);

proofGood = 0;

if sup(r_minus) > 0 
    proofGood = 1
end

end

