function [r_minus, r_plus, proofGood] = kotSchaffer_validateFP(...
          aN, mu, nu, b, b_tail, c, c_tail, b_inf, c_inf, N);

%--------------------------------------------------------------------------
%
%Note: In this file ``the paper'' refers to 
%
%      Fourier-Taylor Approximation of Unstable Manifolds for Compact Maps: 
%          Numerical Implementation and Computer Assisted Error Bounds
%
%
%
%This program implements the validation discussed in Appendix A of the
%paper.  The purpose of the program is simply to check that the hypotheses
%of the lemma are satisfied.  If they are then the program reports that
%the proof has succeded via proofGood = 1, and returns the validated error
%bounds r_minus and r_plus.  These are bounds on the roots of the radius
%polynomial defined in Theorem A.1.
 

%---- Validation: ---------------------------------------------------------


%Initialize an interval version of the identity matrix:
I_N = intval(eye(N+1));
%Build a diagonal matrix representation of b^N.
BN = diag(b);


%Projected Kot-Schaffer differential defined in equation (21)
%This function returns the matrix realization of the differential
DGNaN =  kotSchafferDifferential_logistic_intval(...
    aN, b, c, mu, N) - I_N;

%Projected Kot-Schaffer map defined in equation (20)
GNaN = kotSchaffer_logistic_intval(aN, b, c, mu, N) - aN;

%Approximate numerical inverse of the truncated derivative.
%As defined in the paragraph just before Equation (50) of the 
%present work.
AN = inv(DGNaN.mid);

%Interval representation of the matrix.
AN = intval(AN);

%Product of the turncated differential and the 
%approximate inverse
ANGNaN = AN*GNaN;

%Computation of the matrix norms of A and A dagger.
%Defined just after equation (51)
in_matrixNorm = 0;
A_norm = ellNuOne_MatrixNorm_intval(AN, nu, N);
ANBN_norm = ellNuOne_MatrixNorm_intval(AN*BN, nu, N);
in_matrixNorm = 1;

%Norms of the sequences a, b, c:
a_norm = ellOneNorm_intval(aN, nu, N);
b_N_norm = ellOneNorm_intval(b, nu, N);
b_norm = b_N_norm + b_inf;
c_N_norm = ellOneNorm_intval(c, nu, N);
c_norm = c_N_norm + c_inf;


%-------------------- Start the proof -------------------------------------
%
% These computations simply compute the terms defined in Lemma A.1 of the 
% present work and then check the hypotheses of that Lemma.
%


%Computatoin of Y0 as defined in equation (52):
%The next two lines compute the first and third terms in (52)
Y0 = ellOneNorm_intval(ANGNaN, nu, N);
Y0 = Y0 + abs(mu)*(ANBN_norm + abs(b_tail(N+2)))*(a_norm^2)*c_inf;



cN_pad = intval(zeros(3*N+1,1));
aN_pad = intval(zeros(3*N+1,1));
cN_pad(1:N+1) = c;
aN_pad(1:N+1) = aN;

%Computation of (c^N * a^N * a^N) to order 3N, as needed for the 
%Y0 bound in equation (52).
cNaNaN = int_cubic_sumFFT(cN_pad,aN_pad,aN_pad);



%Computation of the second term in Equation (52):
tailExponents = (N+1:3*N);
nu_tail = nu.^tailExponents;
tempTail = abs(b_tail(N+2:3*N+1).*cNaNaN(N+2:3*N+1).*nu_tail');
sum = intval(ones(1, 2*N))*tempTail;

%Definition of the Y_0 bound via equation (52):
Y0 = Y0 + 2*abs(mu)*sum;


%Computation of the Z_0 bound defined in equation (53):
Z0 = ellNuOne_MatrixNorm_intval(I_N - AN*DGNaN, nu, N);


%Computatoin of the ``kappa bounds'' needed for the Z_1 term
%
% First we need the product of c^N and a^N computed to order 2N:
cN2 = intval(zeros(2*N+1,1));
aN2 = intval(zeros(2*N+1,1));
cN2(1:N+1) = c;
aN2(1:N+1) = aN;
cNaN = convolutionProductCos_intval(cN2, aN2, 2*N);

%Then we can compute the ``kappa bounds''.
%This code computes the constants defined as kappa_n^1 and 
%kappa_n^2 in Lemma A.1.  See also Lemma 3.3.
kappas = kappaBounds(c, aN, nu, N);

%The first term in the expression for Z_1 is a double 
%sum over the entries of A^N and the kappa bounds:
sum = intval(0);

for n = 0:N
    for k = 0:N
        thisTerm = abs(AN(n+1, k+1))*abs(b(k+1))*...
            kappas(k+1)*nu^n;
        sum = sum + thisTerm;
    end
end

%The remainig two terms in the Z_1 bound are simplier:
Z1 = 4*abs(mu)*sum;
Z1 = Z1 + abs(mu)*ANBN_norm*a_norm*c_inf;

%Definition of the Z_1 bound of Lemma A.1:
Z1 = Z1 + abs(mu)*b_tail(N+2)*(1 + 2*c_norm*a_norm);

%Definition of the Z_2 bound of Lemma A.1:
Z2 = 2*abs(mu)*c_norm*ANBN_norm;

%Now check the ``radii-polynomial'' condition:
a_quad = Z2;
b_quad = -(1 - Z0 - Z1);
c_quad = Y0;

%If the proof succedes then r_minus is the tightest error 
%bound given by the contraction mapping argument
r_minus = (-b_quad - sqrt(b_quad^2 - 4*a_quad*c_quad))/(2*a_quad);

%If the proof succedes then r_plus is the largest isolation/uniqueness
%bound given by the contraction mapping argument
r_plus = (-b_quad + sqrt(b_quad^2 - 4*a_quad*c_quad))/(2*a_quad);

proofGood = 0;

%Return the status of the proof:
if sup(r_minus) > 0 
    proofGood = 1;
end

end





