function uniqueCheck = kot_oneUnstableEig_check(...
    p1, r0, Sigma, Q, b_int, b_tail_int, c_int, c_inf_int, mu_int, nu, N)



%--------------------------------------------------------------------------
%
%Note: In this file ``the paper'' refers to 
%
%      Fourier-Taylor Approximation of Unstable Manifolds for Compact Maps: 
%          Numerical Implementation and Computer Assisted Error Bounds
%
%
%
%This program implements the validation discussed in Appendix E of the
%paper.  The purpose of the program is simply to check that the hypotheses
%of the lemma are satisfied.  If they are then the program reports that
%the proof has succeded via uniqueCheck = 1.
 

%---- Validation: ---------------------------------------------------------



aN = p1;
a_star_norm = ellOneNorm_intval(aN, nu, N)+r0;
c_norm = ellOneNorm_intval(c_int, nu, N);

%Validate Eigenvalue/Eigenvector Computatoins for D_intval:
%D_intval is playing the role of A^N in Lemma E.1:

D_intval = kotSchafferDifferential_logistic_intval(...
    p1, b_int, c_int, mu_int, N);

%Compute the diagonalizing transformations in interval arithmetic:
Q_intval = intval(zeros(N+1, N+1));
Sigma_intval = intval(zeros(N+1, N+1));
for n = 0:N
   [Sigma_intval(n+1, n+1), Q_intval(:, n+1)] = ...
         verifyeig(D_intval, Sigma(n+1,n+1), Q(:, n+1)); 
end

Q_inv_intval = inv(Q_intval);

Q_norm_intval = ellNuOne_MatrixNorm_intval(Q_intval, nu, N);
Q_inv_norm_intval = ellNuOne_MatrixNorm_intval(Q_inv_intval, nu, N);

C_eigTest = 1/(1 - abs(Sigma_intval(2,2)));
C_eigTest = C_eigTest*Q_norm_intval*Q_inv_norm_intval;

%Now we have to bound certian truncation errors to measure
%the size of \tilde A.  These computations are discussed in 
%Appendix F of the paper.

%Compute the ``kappa bounds''
kappas = kappaBounds(c_int, p1, nu, N);

sum = intval(0);
sum = sum + abs(b_int(1))*kappas(1);
for n = 1:N
    sum = sum + 2*abs(b_int(n+1))*kappas(n+1)*nu^n;
end

A2_norm = sum;

%Operator Norms:
Ba_star_norm = ellOneNorm_intval(b_int.*aN, nu, N) + ...
                          abs(b_tail_int(N+2))*r0;
BcN_norm = ellOneNorm_intval(b_int.*c_int, nu, N);

A1_norm = 2*mu_int*Ba_star_norm*c_inf_int;
A3_norm = 2*mu_int*BcN_norm*r0;
A4_norm = 2*mu_int*c_norm*a_star_norm*abs(b_tail_int(N+2));

A_norm = A1_norm + A2_norm + A3_norm + A4_norm;

%The constant defined in Lemma E.1.
CondNum_uniqueEig = C_eigTest*A_norm;

%Check that the constant is less than one:
uniqueCheck = 0;
if sup(CondNum_uniqueEig) < 1
    uniqueCheck = 1
end


end



