function [r_min_int, r_max_int, proofGood] = ...
            validateUnstableManifold_Kot1D(...
                    p1_int, r0, r1, PMN, xi_int, lambda_int, ...
                    b_int, b_tail_int, b_inf_int, ...
                    c_int, c_tail_int, c_inf_int, ...
                    mu_int, nu_int, N, M)



%--------------------------------------------------------------------------
%
%
%Note: In this file ``the paper'' refers to 
%
%      Fourier-Taylor Approximation of Unstable Manifolds for Compact Maps: 
%          Numerical Implementation and Computer Assisted Error Bounds
%
%
%
%This program implements the validation discussed in Section 4 of the 
%the paper.  Namely the program computes the constants needed in the 
%statements of the lemmas of Section 4, and then checks the hypotheses
%of the lemmas.  If the hypothese are verrified then the program returns
%success and returns the validated error bounds.  
%
%
%-------------------------------------------------------------------------- 
                
% DOCUMENTATION: This code is nearly identicle to the code in the program
%
% validateUnstableManifold_Kot1D_fast.m
%
%The above mentioned program is carefully documented.  The interested 
%reader should look at that program first for documentation.  The only
%difference is the present program is that a certian bound is computed
%in a sharper, but less fast way.

%==========================================================================
%================== Validate the Parameterization =========================
%==========================================================================


DFPMN = kotSchafferDifferentialMap_P(PMN, M, N, ...
    mid(mu_int), mid(b_int), mid(c_int));

DPhi = PhiDifferential_KotSchaffer(...
        PMN, M, N, mid(p1_int), ...
        mid(lambda_int), mid(xi_int), mid(mu_int), ...
        mid(b_int), mid(c_int));

AMN = PhiDifferentialInverse_KotSchaffer(...
    PMN, DFPMN, M, N, mid(p1_int), ...
    mid(lambda_int), mid(xi_int), mid(mu_int), mid(b_int), mid(c_int));

PMN_bar = intval(PMN);
AMN_int = intval(AMN);

Phi_P = Phi_KotSchaffer_intval(PMN_bar, M, N, p1_int, lambda_int, xi_int, ...
                               mu_int, b_int, c_int);

A_Phi_P = applyOperator_cosineTaylor_LT_intval(AMN, Phi_P, M, N);

Y01 = Xnorm_intval(A_Phi_P, nu_int, M, N);

tic;

Y02 = KotParm_Y02_Bound(PMN_bar, b_int, b_tail_int, c_int, c_tail_int, ...
                                 lambda_int, mu_int, nu_int, M, N);

time_Y02 = toc/60;


tic;
Y03 = KotParm_Y03_Bound(PMN_bar, b_int, b_tail_int, c_int, c_tail_int, ...
                                 lambda_int, mu_int, nu_int, M, N);
time_Y03 = toc/60;


tic;
Y04 = KotParm_Y04_Bound(AMN_int, PMN_bar, b_int, r0, r1, c_inf_int, ...
                                 lambda_int, mu_int, nu_int, M, N);

time_Y04 = toc/60;


Y0 = Y01 + Y02 + Y03 + Y04;


tic;
DPhi_int = PhiDifferential_KotSchaffer_intval(...
                PMN_bar, M, N, lambda_int, mu_int, b_int, c_int);


AMN_DPhiP_int = ...
    composeOperators_cosineTaylor_LT_intval(AMN_int, DPhi_int, M, N);


I_X = intval(zeros(N+1, N+1, M+1, M+1));
for m = 0:M
    I_X(:, :, m+1, m+1) = intval(eye(N+1, N+1));
end

Z0 = X_Operator_norm_intval(I_X - AMN_DPhiP_int, nu_int, M, N);
time_Z0 = toc;



tic;
Z11 = KotParm_Z11_Bound(...
    AMN_int, PMN_bar, b_int, c_int, mu_int, nu_int, M, N);
timeZ11 = toc;


c_norm = ellOneNorm_intval(c_int, nu_int, N) + c_inf_int;
b_norm = ellOneNorm_intval(b_int, nu_int, N) + b_inf_int;
P_norm = Xnorm_intval(PMN_bar, nu_int, M, N);
A_norm = X_Operator_norm_intval(AMN_int, nu_int, M, N);



Z12 = 2*abs(mu_int)*(...
 abs(b_tail_int(N+2))/abs(lambda_int^2 - abs(mu_int)*b_tail_int(N+2)) + ...
   abs(b_int(1))/abs(abs(lambda_int)^(M+1) - abs(mu_int)*abs(b_int(1))) + ...
   abs(b_tail_int(N+2))/abs(abs(lambda_int)^(M+1) - ...
        abs(mu_int)*abs(b_tail_int(N+2))))*c_norm*P_norm;



Z13 = 2*abs(mu_int)*c_inf_int*P_norm*b_norm*A_norm;


Z1 = Z11 + Z12 + Z13;

Z2 = 2*abs(mu_int)*b_norm*c_norm*A_norm;



r_min_int = ((1 - Z1 - Z0) - sqrt((1-Z1-Z0)^2 - 4*Z2*Y0))/(2*Z2);
r_max_int = ((1 - Z1 - Z0) + sqrt((1-Z1-Z0)^2 - 4*Z2*Y0))/(2*Z2);
proofGood = 1;
r_min = sup(r_min_int);

end




