function [r_min_int, r_max_int, proofGood] = ...
            validateUnstableManifold_Kot1D_fast(...
                    p1_int, r0, r1, PMN, xi_int, lambda_int, ...
                    b_int, b_tail_int, b_inf_int, ...
                    c_int, c_tail_int, c_inf_int, ...
                    mu_int, nu_int, N, M)


%--------------------------------------------------------------------------
%
%
%Note: In this file ``the paper'' refers to 
%
%      Fourier-Taylor Approximation of Unstable Manifolds for Compact Maps: 
%          Numerical Implementation and Computer Assisted Error Bounds
%
%
%
%This program implements the validation discussed in Section 4 of the 
%the paper.  Namely the program computes the constants needed in the 
%statements of the lemmas of Section 4, and then checks the hypotheses
%of the lemmas.  If the hypothese are verrified then the program returns
%success and returns the validated error bounds.  
%
%
%-------------------------------------------------------------------------- 


%VALIDATION VALUES: the proof requires some data, which in the paper
%are referred to as ``validation values for the Fourier-Taylor 
%Parameterization of a one dimensional unstable manifold for Kot-Schaffer''
%See defintion 4.5.  Most of these validation values are defined externally
%For example:
%
% -- nu is passed in
% -- mu is passed in
% -- r2 is called locally: c_inf_int  this is passed in
% -- r3 here we use the value of b_{N+1} which is simply
%    stored in the tail information of the variable b_tail_int.
%    In several places below we read out the value of b_tail_int(N+2)
%    in order to obtain the bound r3.
% -- We check externally that N, M, mu, and b satisfy assumption 1
% -- \bar{a}^N and r0 are passed in.  The local names are aN and r0. 
% -- |lambda| > 1 is checked externally.  The local name is lambda_int
% -- \bar{\xi}^N and r1 are passed in.  The local names are xi_int and r1
% -- The operator entries A_{mn}^{MN} are computed internally below. 


%---- Validation: ---------------------------------------------------------
                
                
%==========================================================================
%================== Validate the Parameterization =========================
%==========================================================================


%Kot schaffer differential computed on the level of 
%Fourier-Taylor series
DFPMN = kotSchafferDifferentialMap_P(PMN, M, N, ...
    mid(mu_int), mid(b_int), mid(c_int));

%Truncated Differential of the Phi map for Kot-Schaffer.
%The operator is defined just after Equation (42) in Section 4:
DPhi = PhiDifferential_KotSchaffer(...
        PMN, M, N, mid(p1_int), ...
        mid(lambda_int), mid(xi_int), mid(mu_int), ...
        mid(b_int), mid(c_int));

%The numerical approximate inverse for the truncated differential 
%of Phi.  The inverse computation utilizes the lower triangular 
%structure of DPhi in order to speed up the computations:

AMN = PhiDifferentialInverse_KotSchaffer(...
    PMN, DFPMN, M, N, mid(p1_int), ...
    mid(lambda_int), mid(xi_int), mid(mu_int), mid(b_int), mid(c_int));


PMN_bar = intval(PMN);
AMN_int = intval(AMN);



%-----------------------Lemma 4.6 computations ----------------------------

%The truncated unstable manifold operator for Kot-Schaffer. This 
%is the map defined by Equation (42) in Section 4.

Phi_P = Phi_KotSchaffer_intval(PMN_bar, M, N, p1_int, ...
                               lambda_int, xi_int, ...
                               mu_int, b_int, c_int);

%Computation of the numerical defect term, i.e. apply A^{MN} to 
%Phi^{MN}(P^{MN}).  The application of the operator uses the 
%lower triangular structure in order to speed up the computations
A_Phi_P = applyOperator_cosineTaylor_LT_intval(AMN, Phi_P, M, N);

%Computation of Y_0^1 as in Lemma 4.6:
Y01 = Xnorm_intval(A_Phi_P, nu_int, M, N);



%tic

%Computation of Y_0^2 from Lemma 4.6:
Y02 = KotParm_Y02_Bound_fast(PMN_bar, b_int, b_tail_int, ...
                             c_int, c_tail_int, ...
                             lambda_int, mu_int, nu_int, M, N);

%time_Y02 = toc/60;


%tic

%Computation of Y_0^3 from Lemma 4.6:
Y03 = KotParm_Y03_Bound_fast(PMN_bar, b_int, b_tail_int, ...
                             c_int, c_tail_int, ...
                             lambda_int, mu_int, nu_int, M, N);
                         
%time_Y03 = toc/60;



%tic

%Computation of Y_0^4 from Lemma 4.6:  
%   (We note that \bar A is computed inside the following function)
Y04 = KotParm_Y04_Bound(AMN_int, PMN_bar, b_int, r0, r1, c_inf_int, ...
                                 lambda_int, mu_int, nu_int, M, N);

%time_Y04 = toc/60


%Definition of Y_0 from Lemma 4.6
Y0 = Y01 + Y02 + Y03 + Y04;

%------------------------- Lemma 4.7 computations -------------------------
%tic

%Computation of the interval enclosure of DPhi^{MN}(\bar P)
DPhi_int = PhiDifferential_KotSchaffer_intval(...
                PMN_bar, M, N, lambda_int, mu_int, b_int, c_int);

%Computation of the numerical approximate inverse A^{MN} of DPhi from 
%above

AMN_DPhiP_int = ...
    composeOperators_cosineTaylor_LT_intval(AMN_int, DPhi_int, M, N);


%Definition of the interval identity operator
I_X = intval(zeros(N+1, N+1, M+1, M+1));
for m = 0:M
    I_X(:, :, m+1, m+1) = intval(eye(N+1, N+1));
end

%Computation of Z_0 from Lemma 4.7
Z0 = X_Operator_norm_intval(I_X - AMN_DPhiP_int, nu_int, M, N);
%time_Z0 = toc



%---------------------------- Lemma 4.8 bounds ----------------------------

%Computaiton of the Z_1^1 bound of Lemma 4.8:
% The necessary kappa bounds are computed inside the function
%tic
Z11 = KotParm_Z11_Bound_fast(...
    AMN_int, PMN_bar, b_int, c_int, mu_int, nu_int, M, N);
%timeZ11 = toc

%Prep to compute Z_1^2:
c_norm = ellOneNorm_intval(c_int, nu_int, N) + c_inf_int;
b_norm = ellOneNorm_intval(b_int, nu_int, N) + b_inf_int;
P_norm = Xnorm_intval(PMN_bar, nu_int, M, N);
A_norm = X_Operator_norm_intval(AMN_int, nu_int, M, N);


%Compute Z_1^2 from Lemma 4.8
Z12 = 2*abs(mu_int)*(...
 abs(b_tail_int(N+2))/abs(lambda_int^2 - abs(mu_int)*b_tail_int(N+2)) + ...
   abs(b_int(1))/abs(abs(lambda_int)^(M+1) - abs(mu_int)*abs(b_int(1))) + ...
   abs(b_tail_int(N+2))/abs(abs(lambda_int)^(M+1) - ...
        abs(mu_int)*abs(b_tail_int(N+2))))*c_norm*P_norm;


%Compute Z_-^3 from Lemma 4.8
Z13 = 2*abs(mu_int)*c_inf_int*P_norm*b_norm*A_norm;

%Define Z_1 from Lemma 4.8:
Z1 = Z11 + Z12 + Z13;

%--------------------------- Lemma 4.9 bounds -----------------------------

%Define the Z_2 bound from Lemma 4.9:
Z2 = 2*abs(mu_int)*b_norm*c_norm*A_norm;


%--------------------------Lemma 4.10 -------------------------------------

Y0
Z0
Z1
Z2

%Compute the roots of the radii polynomial of lemma 4.10:
r_min_int = ((1 - Z1 - Z0) - sqrt((1-Z1-Z0)^2 - 4*Z2*Y0))/(2*Z2);
r_max_int = ((1 - Z1 - Z0) + sqrt((1-Z1-Z0)^2 - 4*Z2*Y0))/(2*Z2);
proofGood = 0;

%Return the status of the proof:
if sup(r_min_int) > 0 
    proofGood = 1;
end



end

